@@ -1,7 +1,7 @@
 package Array::Set;
 
-our $DATE = '2015-11-18'; # DATE
-our $VERSION = '0.02'; # VERSION
+our $DATE = '2016-09-15'; # DATE
+our $VERSION = '0.03'; # VERSION
 
 use 5.010001;
 use strict;
@@ -176,7 +176,23 @@ sub _doit {
 }
 
 sub set_diff {
-    _doit('diff', @_);
+    my $opts = ref($_[0]) eq 'HASH' ? shift : {};
+    if ($opts->{ignore_case} || $opts->{ignore_blanks}) {
+        _doit('diff', $opts, @_);
+    } else {
+        # fast version, without ib/ic
+        my $set1 = shift;
+        my $res;
+        while (@_) {
+            my %set2 = map { $_=>1 } @{ shift @_ };
+            $res = [];
+            for my $el (@$set1) {
+                push @$res, $el unless $set2{$el};
+            }
+            $set1 = $res;
+        }
+        $res;
+    }
 }
 
 sub set_symdiff {
@@ -188,7 +204,23 @@ sub set_union {
 }
 
 sub set_intersect {
-    _doit('intersect', @_);
+    my $opts = ref($_[0]) eq 'HASH' ? shift : {};
+    if ($opts->{ignore_case} || $opts->{ignore_blanks}) {
+        _doit('intersect', $opts, @_);
+    } else {
+        # fast version, without ib/ic
+        my $set1 = shift;
+        my $res;
+        while (@_) {
+            my %set2 = map { $_=>1 } @{ shift @_ };
+            $res = [];
+            for my $el (@$set1) {
+                push @$res, $el if $set2{$el};
+            }
+            $set1 = $res;
+        }
+        $res;
+    }
 }
 
 1;
@@ -206,7 +238,7 @@ Array::Set - Perform set operations on arrays
 
 =head1 VERSION
 
-This document describes version 0.02 of Array::Set (from Perl distribution Array-Set), released on 2015-11-18.
+This document describes version 0.03 of Array::Set (from Perl distribution Array-Set), released on 2016-09-15.
 
 =head1 SYNOPSIS
 
@@ -214,7 +246,7 @@ This document describes version 0.02 of Array::Set (from Perl distribution Array
 
  set_diff([1,2,3,4], [2,3,4,5]);            # => [1]
  set_diff([1,2,3,4], [2,3,4,5], [3,4,5,6]); # => [1]
- set_diff({ci=>1}, ["a","b"], ["B","c"]);   # => ["a"]
+ set_diff({ignore_case=>1}, ["a","b"], ["B","c"]);   # => ["a"]
 
  set_symdiff([1,2,3,4], [2,3,4,5]);            # => [1,5]
  set_symdiff([1,2,3,4], [2,3,4,5], [3,4,5,6]); # => [1,6]
@@ -248,11 +280,10 @@ Characteristics and differences with other similar modules:
 
 =back
 
-B<NOTE: This early release is a pretty direct port/translation from App::setop,
-and is not as optimized as I'd like it to be. More optimizations will be done in
-the future.> If you want more speed, I'd recommend L<Set::Object> (also supports
-references/objects and performing operations on 3+ sets, XS) or simply
-L<List::MoreUtils> (very popular module, offers XS version).
+B<NOTE:> A couple of operations are not optimized yet. More optimizations will
+be done in the future. See some benchmarks in L<Bencher::Scenarios::ArraySet>
+distribution. I also recommend L<Set::Object> (also supports references/objects,
+XS) or simply L<List::MoreUtils> (very popular module, offers XS version).
 
 =head1 FUNCTIONS
 
@@ -292,14 +323,6 @@ known options.
 Perform intersection (find elements common in all the sets). Accept optional
 hashref as the first argument for options. See C<set_diff> for known options.
 
-=head1 SEE ALSO
-
-L<App::setop> to perform set operations on lines of files on the command-line.
-
-L<Array::Utils>, L<Set::Scalar>, L<List::MoreUtils> (C<uniq> for union,
-C<singleton> for symmetric diff), L<Set::Array>, L<Array::AsObject>,
-L<Set::Object>, L<Set::Tiny>.
-
 =head1 HOMEPAGE
 
 Please visit the project's homepage at L<https://metacpan.org/release/Array-Set>.
@@ -316,13 +339,21 @@ When submitting a bug or request, please include a test-file or a
 patch to an existing test-file that illustrates the bug or desired
 feature.
 
+=head1 SEE ALSO
+
+L<App::setop> to perform set operations on lines of files on the command-line.
+
+L<Array::Utils>, L<Set::Scalar>, L<List::MoreUtils> (C<uniq> for union,
+C<singleton> for symmetric diff), L<Set::Array>, L<Array::AsObject>,
+L<Set::Object>, L<Set::Tiny>.
+
 =head1 AUTHOR
 
 perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2015 by perlancar@cpan.org.
+This software is copyright (c) 2016 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
